@@ -1,45 +1,33 @@
 import asyncio
-from weakref import ref
 from decimal import Decimal
-import re
 import threading
-import traceback, sys
 from typing import TYPE_CHECKING, List, Optional, Dict, Any
+from urllib.parse import urlparse
 
 from kivy.app import App
-from kivy.cache import Cache
 from kivy.clock import Clock
-from kivy.compat import string_types
-from kivy.properties import (ObjectProperty, DictProperty, NumericProperty,
-                             ListProperty, StringProperty)
-
-from kivy.uix.recycleview import RecycleView
-from kivy.uix.label import Label
-from kivy.uix.behaviors import ToggleButtonBehavior
-from kivy.uix.image import Image
-
+from kivy.properties import ObjectProperty
 from kivy.lang import Builder
 from kivy.factory import Factory
-from kivy.utils import platform
+from kivy.uix.recycleview import RecycleView
+from kivy.properties import StringProperty
 
-from electrum.util import profiler, parse_URI, format_time, InvalidPassword, NotEnoughFunds, Fiat
-from electrum.invoices import (PR_TYPE_ONCHAIN, PR_TYPE_LN, PR_DEFAULT_EXPIRATION_WHEN_CREATING,
+from electrum.invoices import (PR_DEFAULT_EXPIRATION_WHEN_CREATING,
                                PR_PAID, PR_UNKNOWN, PR_EXPIRED, PR_INFLIGHT,
-                               LNInvoice, pr_expiration_values, Invoice, OnchainInvoice)
+                               pr_expiration_values, Invoice)
 from electrum import bitcoin, constants
-from electrum.transaction import Transaction, tx_from_any, PartialTransaction, PartialTxOutput
-from electrum.util import parse_URI, InvalidBitcoinURI, TxMinedInfo, maybe_extract_bolt11_invoice
-from electrum.plugin import run_hook
-from electrum.wallet import InternalAddressCorruption
-from electrum import simple_config
-from electrum.simple_config import FEERATE_WARNING_HIGH_FEE, FEE_RATIO_HIGH_WARNING
-from electrum.lnaddr import lndecode
-from electrum.lnutil import RECEIVED, SENT, PaymentFailure
+from electrum import lnutil
+from electrum.transaction import tx_from_any, PartialTxOutput
+from electrum.util import (parse_URI, InvalidBitcoinURI, TxMinedInfo, maybe_extract_lightning_payment_identifier,
+                           InvoiceError, format_time, parse_max_spend, BITCOIN_BIP21_URI_SCHEME)
+from electrum.lnaddr import lndecode, LnInvoiceException
+from electrum.lnurl import decode_lnurl, request_lnurl, callback_lnurl, LNURLError, LNURL6Data
 from electrum.logging import Logger
+from electrum.network import Network
 
-from .dialogs.question import Question
-from .dialogs.lightning_open_channel import LightningOpenChannelDialog
+from .dialogs.confirm_tx_dialog import ConfirmTxDialog
 
+from electrum.gui.kivy import KIVY_GUI_PATH
 from electrum.gui.kivy.i18n import _
 
 if TYPE_CHECKING:
@@ -96,9 +84,9 @@ TX_ICONS = [
 ]
 
 
-Builder.load_file('electrum/gui/kivy/uix/ui_screens/history.kv')
-Builder.load_file('electrum/gui/kivy/uix/ui_screens/send.kv')
-Builder.load_file('electrum/gui/kivy/uix/ui_screens/receive.kv')
+Builder.load_file(KIVY_GUI_PATH + '/uix/ui_screens/history.kv')
+Builder.load_file(KIVY_GUI_PATH + '/uix/ui_screens/send.kv')
+Builder.load_file(KIVY_GUI_PATH + '/uix/ui_screens/receive.kv')
 
 
 class HistoryScreen(CScreen):
@@ -118,7 +106,7 @@ class HistoryScreen(CScreen):
             self.app.lightning_tx_dialog(tx_item)
             return
         if tx_item.get('lightning'):
-            tx = self.app.wallet.lnworker.lnwatcher.db.get_transaction(key)
+            tx = self.app.wallet.adb.get_transaction(key)
         else:
             tx = self.app.wallet.db.get_transaction(key)
         if not tx:
@@ -130,21 +118,19 @@ class HistoryScreen(CScreen):
         timestamp = tx_item['timestamp']
         key = tx_item.get('txid') or tx_item['payment_hash']
         if is_lightning:
-            status = 0
             status_str = 'unconfirmed' if timestamp is None else format_time(int(timestamp))
-            icon = "atlas://electrum/gui/kivy/theming/light/lightning"
+            icon = f'atlas://{KIVY_GUI_PATH}/theming/atlas/light/lightning'
             message = tx_item['label']
             fee_msat = tx_item['fee_msat']
             fee = int(fee_msat/1000) if fee_msat else None
             fee_text = '' if fee is None else 'fee: %d sat'%fee
         else:
             tx_hash = tx_item['txid']
-            conf = tx_item['confirmations']
             tx_mined_info = TxMinedInfo(height=tx_item['height'],
                                         conf=tx_item['confirmations'],
                                         timestamp=tx_item['timestamp'])
             status, status_str = self.app.wallet.get_tx_status(tx_hash, tx_mined_info)
-            icon = "atlas://electrum/gui/kivy/theming/light/" + TX_ICONS[status]
+            icon = f'atlas://{KIVY_GUI_PATH}/theming/atlas/light/' + TX_ICONS[status]
             message = tx_item['label'] or tx_hash
             fee = tx_item['fee_sat']
             fee_text = '' if fee is None else 'fee: %d sat'%fee
@@ -158,9 +144,11 @@ class HistoryScreen(CScreen):
         value = tx_item['value'].value
         if value is not None:
             ri['is_mine'] = value <= 0
-            ri['amount'] = self.app.format_amount(value, is_diff = True)
+            ri['amount'] = self.app.format_amount(value, is_diff=True)
+            ri['base_unit'] = self.app.base_unit
             if 'fiat_value' in tx_item:
                 ri['quote_text'] = str(tx_item['fiat_value'])
+                ri['fx_ccy'] = tx_item['fiat_value'].ccy
         return ri
 
     def update(self, see_all=False):
@@ -178,15 +166,44 @@ class SendScreen(CScreen, Logger):
     kvname = 'send'
     payment_request = None  # type: Optional[PaymentRequest]
     parsed_URI = None
+    lnurl_data = None  # type: Optional[LNURL6Data]
 
     def __init__(self, **kwargs):
         CScreen.__init__(self, **kwargs)
         Logger.__init__(self)
         self.is_max = False
+        # note: most the fields get declared in send.kv, this way they are kivy Properties
 
     def set_URI(self, text: str):
+        """Takes
+        Lightning identifiers:
+        * lightning-URI (containing bolt11 or lnurl)
+        * bolt11 invoice
+        * lnurl
+        Bitcoin identifiers:
+        * bitcoin-URI
+        * bitcoin address
+        and sets the sending screen.
+
+        TODO maybe rename method...
+        """
         if not self.app.wallet:
             return
+        text = text.strip()
+        if not text:
+            return
+        if invoice_or_lnurl := maybe_extract_lightning_payment_identifier(text):
+            if invoice_or_lnurl.startswith('lnurl'):
+                self.set_lnurl6(invoice_or_lnurl)
+            else:
+                self.set_bolt11(invoice_or_lnurl)
+        elif text.lower().startswith(BITCOIN_BIP21_URI_SCHEME + ':') or bitcoin.is_address(text):
+            self.set_bip21(text)
+        else:
+            self.app.show_error(f"Failed to parse text: {text[:10]}...")
+            return
+
+    def set_bip21(self, text: str):
         try:
             uri = parse_URI(text, self.app.on_pr, loop=self.app.asyncio_loop)
         except InvalidBitcoinURI as e:
@@ -201,46 +218,69 @@ class SendScreen(CScreen, Logger):
         self.payment_request = None
         self.is_lightning = False
 
-    def set_ln_invoice(self, invoice: str):
+    def set_bolt11(self, invoice: str):
         try:
             invoice = str(invoice).lower()
-            lnaddr = lndecode(invoice, expected_hrp=constants.net.SEGWIT_HRP)
-        except Exception as e:
-            self.app.show_info(invoice + _(" is not a valid Lightning invoice: ") + repr(e)) # repr because str(Exception()) == ''
+            lnaddr = lndecode(invoice)
+        except LnInvoiceException as e:
+            self.app.show_info(_("Invoice is not a valid Lightning invoice: ") + repr(e)) # repr because str(Exception()) == ''
+            return
+        except lnutil.IncompatibleOrInsaneFeatures as e:
+            self.app.show_info(_("Invoice requires unknown or incompatible Lightning feature") + f":\n{e!r}")
             return
         self.address = invoice
-        self.message = dict(lnaddr.tags).get('d', None)
+        self.message = lnaddr.get_description()
         self.amount = self.app.format_amount_and_units(lnaddr.amount * bitcoin.COIN) if lnaddr.amount else ''
         self.payment_request = None
         self.is_lightning = True
 
+    def set_lnurl6(self, lnurl: str):
+        url = decode_lnurl(lnurl)
+        domain = urlparse(url).netloc
+        # FIXME network request blocking GUI thread:
+        lnurl_data = Network.run_from_another_thread(request_lnurl(url))
+        if not lnurl_data:
+            return
+        self.lnurl_data = lnurl_data
+        self.address = "invoice from lnurl"
+        self.message = f"lnurl: {domain}: {lnurl_data.metadata_plaintext}"
+        self.amount = self.app.format_amount_and_units(lnurl_data.min_sendable_sat)
+        self.is_lightning = True
+        self.is_lnurl = True  # `bool(self.lnurl_data)` should be equivalent, this is only here as it is a kivy Property
+
     def update(self):
         if self.app.wallet is None:
             return
-        _list = self.app.wallet.get_invoices()
+        _list = self.app.wallet.get_unpaid_invoices()
         _list.reverse()
         payments_container = self.ids.payments_container
-        payments_container.data = [self.get_card(item) for item in _list]
+        payments_container.data = [self.get_card(invoice) for invoice in _list]
+
+    def update_item(self, key, invoice):
+        payments_container = self.ids.payments_container
+        data = payments_container.data
+        for item in data:
+            if item['key'] == key:
+                item.update(self.get_card(invoice))
+        payments_container.data = data
+        payments_container.refresh_from_data()
 
     def show_item(self, obj):
-        self.app.show_invoice(obj.is_lightning, obj.key)
+        self.app.show_invoice(obj.key)
 
-    def get_card(self, item: Invoice):
+    def get_card(self, item: Invoice) -> Dict[str, Any]:
         status = self.app.wallet.get_invoice_status(item)
         status_str = item.get_status_str(status)
-        is_lightning = item.type == PR_TYPE_LN
+        is_lightning = item.is_lightning()
+        key = self.app.wallet.get_key_for_outgoing_invoice(item)
         if is_lightning:
-            assert isinstance(item, LNInvoice)
-            key = item.rhash
-            address = key
+            address = item.rhash
             if self.app.wallet.lnworker:
                 log = self.app.wallet.lnworker.logs.get(key)
                 if status == PR_INFLIGHT and log:
                     status_str += '... (%d)'%len(log)
             is_bip70 = False
         else:
-            assert isinstance(item, OnchainInvoice)
-            key = item.id
             address = item.get_address()
             is_bip70 = bool(item.bip70)
         return {
@@ -264,6 +304,8 @@ class SendScreen(CScreen, Logger):
         self.is_bip70 = False
         self.parsed_URI = None
         self.is_max = False
+        self.lnurl_data = None
+        self.is_lnurl = False
 
     def set_request(self, pr: 'PaymentRequest'):
         self.address = pr.get_requestor()
@@ -278,21 +320,7 @@ class SendScreen(CScreen, Logger):
         if not data:
             self.app.show_info(_("Clipboard is empty"))
             return
-        # try to decode as transaction
-        try:
-            tx = tx_from_any(data)
-            tx.deserialize()
-        except:
-            tx = None
-        if tx:
-            self.app.tx_dialog(tx)
-            return
-        # try to decode as URI/address
-        bolt11_invoice = maybe_extract_bolt11_invoice(data)
-        if bolt11_invoice is not None:
-            self.set_ln_invoice(bolt11_invoice)
-        else:
-            self.set_URI(data)
+        self.app.on_data_input(data)
 
     def read_invoice(self):
         address = str(self.address)
@@ -303,29 +331,37 @@ class SendScreen(CScreen, Logger):
             self.app.show_error(_('Please enter an amount'))
             return
         if self.is_max:
-            amount = '!'
+            amount_sat = '!'
         else:
             try:
-                amount = self.app.get_amount(self.amount)
+                amount_sat = self.app.get_amount(self.amount)
             except:
                 self.app.show_error(_('Invalid amount') + ':\n' + self.amount)
                 return
         message = self.message
-        if self.is_lightning:
-            return LNInvoice.from_bech32(address)
-        else:  # on-chain
-            if self.payment_request:
-                outputs = self.payment_request.get_outputs()
+        try:
+            if self.is_lightning:
+                assert type(amount_sat) is int
+                invoice = Invoice.from_bech32(address)
+                if invoice.amount_msat is None:
+                    invoice.amount_msat = int(amount_sat * 1000)
+                return invoice
             else:
-                if not bitcoin.is_address(address):
-                    self.app.show_error(_('Invalid Bitcoin Address') + ':\n' + address)
-                    return
-                outputs = [PartialTxOutput.from_address_and_value(address, amount)]
-            return self.app.wallet.create_invoice(
-                outputs=outputs,
-                message=message,
-                pr=self.payment_request,
-                URI=self.parsed_URI)
+                # on-chain
+                if self.payment_request:
+                    outputs = self.payment_request.get_outputs()
+                else:
+                    if not bitcoin.is_address(address):
+                        self.app.show_error(_('Invalid Bitcoin Address') + ':\n' + address)
+                        return
+                    outputs = [PartialTxOutput.from_address_and_value(address, amount_sat)]
+                return self.app.wallet.create_invoice(
+                    outputs=outputs,
+                    message=message,
+                    pr=self.payment_request,
+                    URI=self.parsed_URI)
+        except InvoiceError as e:
+            self.app.show_error(_('Error creating payment') + ':\n' + str(e))
 
     def do_save(self):
         invoice = self.read_invoice()
@@ -338,7 +374,35 @@ class SendScreen(CScreen, Logger):
         self.do_clear()
         self.update()
 
+    def _lnurl_get_invoice(self) -> None:
+        assert self.lnurl_data
+        try:
+            amount = self.app.get_amount(self.amount)
+        except:
+            self.app.show_error(_('Invalid amount') + ':\n' + self.amount)
+            return
+        if not (self.lnurl_data.min_sendable_sat <= amount <= self.lnurl_data.max_sendable_sat):
+            self.app.show_error(f'Amount must be between {self.lnurl_data.min_sendable_sat} and {self.lnurl_data.max_sendable_sat} sat.')
+            return
+        try:
+            # FIXME network request blocking GUI thread:
+            invoice_data = Network.run_from_another_thread(callback_lnurl(
+                self.lnurl_data.callback_url,
+                params={'amount': amount * 1000},
+            ))
+        except LNURLError as e:
+            self.app.show_error(f"LNURL request encountered error: {e}")
+            self.do_clear()
+            return
+        invoice = invoice_data.get('pr')
+        self.set_bolt11(invoice)
+        self.lnurl_data = None
+        self.is_lnurl = False
+
     def do_pay(self):
+        if self.lnurl_data:
+            self._lnurl_get_invoice()
+            return
         invoice = self.read_invoice()
         if not invoice:
             return
@@ -347,62 +411,34 @@ class SendScreen(CScreen, Logger):
     def do_pay_invoice(self, invoice):
         if invoice.is_lightning():
             if self.app.wallet.lnworker:
-                self.app.protected(_('Pay lightning invoice?'), self._do_pay_lightning, (invoice,))
+                amount_sat = invoice.get_amount_sat()
+                msg = _("Pay lightning invoice?") + '\n\n' + _("This will send {}?").format(self.app.format_amount_and_units_with_fiat(amount_sat)) +'\n'
+                self.app.protected(msg, self._do_pay_lightning, (invoice,))
             else:
                 self.app.show_error(_("Lightning payments are not available for this wallet"))
         else:
-            do_pay = lambda rbf: self._do_pay_onchain(invoice, rbf)
-            if self.app.electrum_config.get('use_rbf'):
-                d = Question(_('Should this transaction be replaceable?'), do_pay)
-                d.open()
-            else:
-                do_pay(False)
+            self._do_pay_onchain(invoice)
 
-    def _do_pay_lightning(self, invoice: LNInvoice, pw) -> None:
+    def _do_pay_lightning(self, invoice: Invoice, pw) -> None:
+        amount_msat = invoice.get_amount_msat()
+        def pay_thread():
+            try:
+                coro = self.app.wallet.lnworker.pay_invoice(invoice.lightning_invoice, amount_msat=amount_msat)
+                fut = asyncio.run_coroutine_threadsafe(coro, self.app.network.asyncio_loop)
+                fut.result()
+            except Exception as e:
+                self.app.show_error(repr(e))
         self.save_invoice(invoice)
-        threading.Thread(
-            target=self.app.wallet.lnworker.pay,
-            args=(invoice.invoice,),
-            kwargs={
-                'attempts': 10,
-            },
-        ).start()
+        threading.Thread(target=pay_thread).start()
 
-    def _do_pay_onchain(self, invoice: OnchainInvoice, rbf: bool) -> None:
-        # make unsigned transaction
+    def _do_pay_onchain(self, invoice: Invoice) -> None:
         outputs = invoice.outputs
+        amount = sum(map(lambda x: x.value, outputs)) if not any(parse_max_spend(x.value) for x in outputs) else '!'
         coins = self.app.wallet.get_spendable_coins(None)
-        try:
-            tx = self.app.wallet.make_unsigned_transaction(coins=coins, outputs=outputs)
-        except NotEnoughFunds:
-            self.app.show_error(_("Not enough funds"))
-            return
-        except Exception as e:
-            self.logger.exception('')
-            self.app.show_error(repr(e))
-            return
-        if rbf:
-            tx.set_rbf(True)
-        fee = tx.get_fee()
-        amount = sum(map(lambda x: x.value, outputs)) if '!' not in [x.value for x in outputs] else tx.output_value()
-        msg = [
-            _("Amount to be sent") + ": " + self.app.format_amount_and_units(amount),
-            _("Mining fee") + ": " + self.app.format_amount_and_units(fee),
-        ]
-        x_fee = run_hook('get_tx_extra_fee', self.app.wallet, tx)
-        if x_fee:
-            x_fee_address, x_fee_amount = x_fee
-            msg.append(_("Additional fees") + ": " + self.app.format_amount_and_units(x_fee_amount))
-
-        feerate = Decimal(fee) / tx.estimated_size()  # sat/byte
-        fee_ratio = Decimal(fee) / amount if amount else 1
-        if fee_ratio >= FEE_RATIO_HIGH_WARNING:
-            msg.append(_('Warning') + ': ' + _("The fee for this transaction seems unusually high.")
-                       + f' ({fee_ratio*100:.2f}% of amount)')
-        elif feerate > FEERATE_WARNING_HIGH_FEE / 1000:
-            msg.append(_('Warning') + ': ' + _("The fee for this transaction seems unusually high.")
-                       + f' (feerate: {feerate:.2f} sat/byte)')
-        self.app.protected('\n'.join(msg), self.send_tx, (tx, invoice))
+        make_tx = lambda rbf: self.app.wallet.make_unsigned_transaction(coins=coins, outputs=outputs, rbf=rbf)
+        on_pay = lambda tx: self.app.protected(_('Send payment?'), self.send_tx, (tx, invoice))
+        d = ConfirmTxDialog(self.app, amount=amount, make_tx=make_tx, on_pay=on_pay)
+        d.open()
 
     def send_tx(self, tx, invoice, password):
         if self.app.wallet.has_password() and password is None:
@@ -421,29 +457,21 @@ class SendScreen(CScreen, Logger):
         else:
             self.app.tx_dialog(tx)
 
-    def clear_invoices_dialog(self):
-        invoices = self.app.wallet.get_invoices()
-        if not invoices:
-            return
-        def callback(c):
-            if c:
-                for req in invoices:
-                    key = req.rhash if req.is_lightning() else req.get_address()
-                    self.app.wallet.delete_invoice(key)
-                self.update()
-        n = len(invoices)
-        d = Question(_('Delete {} invoices?').format(n), callback)
-        d.open()
-
 
 class ReceiveScreen(CScreen):
 
     kvname = 'receive'
+    expiration_text = StringProperty('')
 
     def __init__(self, **kwargs):
         super(ReceiveScreen, self).__init__(**kwargs)
         Clock.schedule_interval(lambda dt: self.update(), 5)
         self.is_max = False # not used for receiving (see app.amount_dialog)
+        self.expiration_text = pr_expiration_values[self.expiry()]
+
+    def on_open(self):
+        c = self.expiry()
+        self.expiration_text = pr_expiration_values[c]
 
     def expiry(self):
         return self.app.electrum_config.get('request_expiry', PR_DEFAULT_EXPIRATION_WHEN_CREATING)
@@ -469,11 +497,7 @@ class ReceiveScreen(CScreen):
 
     def get_URI(self):
         from electrum.util import create_bip21_uri
-        amount = self.amount
-        if amount:
-            a, u = self.amount.split()
-            assert u == self.app.base_unit
-            amount = Decimal(a) * pow(10, self.app.decimal_point())
+        amount = self.app.get_amount(self.amount)
         return create_bip21_uri(self.address, amount, self.message)
 
     def do_copy(self):
@@ -481,35 +505,41 @@ class ReceiveScreen(CScreen):
         self.app._clipboard.copy(uri)
         self.app.show_info(_('Request copied to clipboard'))
 
-    def new_request(self, lightning):
-        amount = self.amount
-        amount = self.app.get_amount(amount) if amount else 0
+    def new_request(self):
+        amount_str = self.amount
+        amount_sat = self.app.get_amount(amount_str) if amount_str else 0
         message = self.message
-        if lightning:
-            key = self.app.wallet.lnworker.add_request(amount, message, self.expiry())
+        expiry = self.expiry()
+        if amount_sat and amount_sat < self.app.wallet.dust_threshold():
+            self.address = ''
+            if not self.app.wallet.has_lightning():
+                self.app.show_info(_('Amount too small to be received onchain'))
+                return
         else:
             addr = self.address or self.app.wallet.get_unused_address()
             if not addr:
-                self.app.show_info(_('No address available. Please remove some of your pending requests.'))
-                return
+                if not self.app.wallet.is_deterministic():
+                    addr = self.app.wallet.get_receiving_address()
+                else:
+                    self.app.show_info(_('No address available. Please remove some of your pending requests.'))
+                    return
             self.address = addr
-            req = self.app.wallet.make_payment_request(addr, amount, message, self.expiry())
-            self.app.wallet.add_payment_request(req)
-            key = addr
+        try:
+            key = self.app.wallet.create_request(amount_sat, message, expiry, self.address)
+        except InvoiceError as e:
+            self.app.show_error(_('Error creating payment request') + ':\n' + str(e))
+            return
         self.clear()
         self.update()
-        self.app.show_request(lightning, key)
+        self.app.show_request(key)
 
     def get_card(self, req: Invoice) -> Dict[str, Any]:
         is_lightning = req.is_lightning()
         if not is_lightning:
-            assert isinstance(req, OnchainInvoice)
             address = req.get_address()
-            key = address
         else:
-            assert isinstance(req, LNInvoice)
-            key = req.rhash
-            address = req.invoice
+            address = req.lightning_invoice
+        key = self.app.wallet.get_key_for_receive_request(req)
         amount = req.get_amount_sat()
         description = req.message
         status = self.app.wallet.get_request_status(key)
@@ -528,34 +558,34 @@ class ReceiveScreen(CScreen):
     def update(self):
         if self.app.wallet is None:
             return
-        _list = self.app.wallet.get_sorted_requests()
+        _list = self.app.wallet.get_unpaid_requests()
         _list.reverse()
         requests_container = self.ids.requests_container
         requests_container.data = [self.get_card(item) for item in _list]
 
+    def update_item(self, key, request):
+        payments_container = self.ids.requests_container
+        data = payments_container.data
+        for item in data:
+            if item['key'] == key:
+                status = self.app.wallet.get_request_status(key)
+                status_str = request.get_status_str(status)
+                item['status'] = status
+                item['status_str'] = status_str
+        payments_container.data = data # needed?
+        payments_container.refresh_from_data()
+
     def show_item(self, obj):
-        self.app.show_request(obj.is_lightning, obj.key)
+        self.app.show_request(obj.key)
 
     def expiration_dialog(self, obj):
         from .dialogs.choice_dialog import ChoiceDialog
         def callback(c):
             self.app.electrum_config.set_key('request_expiry', c)
+            self.expiration_text = pr_expiration_values[c]
         d = ChoiceDialog(_('Expiration date'), pr_expiration_values, self.expiry(), callback)
         d.open()
 
-    def clear_requests_dialog(self):
-        requests = self.app.wallet.get_sorted_requests()
-        if not requests:
-            return
-        def callback(c):
-            if c:
-                self.app.wallet.clear_requests()
-                self.update()
-        n = len(requests)
-        d = Question(_('Delete {} requests?').format(n), callback)
-        d.open()
-
-
 
 class TabbedCarousel(Factory.TabbedPanel):
     '''Custom TabbedPanel using a carousel used in the Main Screen
