@@ -144,8 +144,14 @@ class NoDynamicFeeEstimates(Exception):
 
 
 class InvalidPassword(Exception):
+    def __init__(self, message: Optional[str] = None):
+        self.message = message
+
     def __str__(self):
-        return _("Incorrect password")
+        if self.message is None:
+            return _("Incorrect password")
+        else:
+            return str(self.message)
 
 
 class AddTransactionException(Exception):
@@ -434,7 +440,7 @@ def profiler(func):
         t0 = time.time()
         o = func(*args, **kw_args)
         t = time.time() - t0
-        _profiler_logger.debug(f"{name} {t:,.4f}")
+        _profiler_logger.debug(f"{name} {t:,.4f} sec")
         return o
     return lambda *args, **kw_args: do_profile(args, kw_args)
 
@@ -444,7 +450,8 @@ def android_ext_dir():
     return primary_external_storage_path()
 
 def android_backup_dir():
-    d = os.path.join(android_ext_dir(), 'org.electrum.electrum')
+    pkgname = get_android_package_name()
+    d = os.path.join(android_ext_dir(), pkgname)
     if not os.path.exists(d):
         os.mkdir(d)
     return d
@@ -511,6 +518,26 @@ def get_new_wallet_name(wallet_folder: str) -> str:
     return filename
 
 
+def is_android_debug_apk() -> bool:
+    is_android = 'ANDROID_DATA' in os.environ
+    if not is_android:
+        return False
+    from jnius import autoclass
+    pkgname = get_android_package_name()
+    build_config = autoclass(f"{pkgname}.BuildConfig")
+    return bool(build_config.DEBUG)
+
+
+def get_android_package_name() -> str:
+    is_android = 'ANDROID_DATA' in os.environ
+    assert is_android
+    from jnius import autoclass
+    from android.config import ACTIVITY_CLASS_NAME
+    activity = autoclass(ACTIVITY_CLASS_NAME).mActivity
+    pkgname = str(activity.getPackageName())
+    return pkgname
+
+
 def assert_bytes(*args):
     """
     porting helper, assert args type
@@ -942,6 +969,7 @@ def parse_URI(uri: str, on_pr: Callable = None, *, loop=None) -> dict:
     """Raises InvalidBitcoinURI on malformed URI."""
     from . import bitcoin
     from .bitcoin import COIN, TOTAL_COIN_SUPPLY_LIMIT_IN_BTC
+    from .lnaddr import lndecode
 
     if not isinstance(uri, str):
         raise InvalidBitcoinURI(f"expected string, not {repr(uri)}")
@@ -1004,6 +1032,17 @@ def parse_URI(uri: str, on_pr: Callable = None, *, loop=None) -> dict:
             out['sig'] = bh2u(bitcoin.base_decode(out['sig'], base=58))
         except Exception as e:
             raise InvalidBitcoinURI(f"failed to parse 'sig' field: {repr(e)}") from e
+    if 'lightning' in out:
+        try:
+            lnaddr = lndecode(out['lightning'])
+            amount_sat = out.get('amount')
+            if amount_sat:
+                assert int(lnaddr.get_amount_sat()) == amount_sat
+            address = out.get('address')
+            if address:
+                assert lnaddr.get_fallback_address() == address
+        except Exception as e:
+            raise InvalidBitcoinURI(f"Inconsistent lightning field: {repr(e)}") from e
 
     r = out.get('r')
     sig = out.get('sig')
@@ -1019,7 +1058,7 @@ def parse_URI(uri: str, on_pr: Callable = None, *, loop=None) -> dict:
                 request = await pr.get_payment_request(r)
             if on_pr:
                 on_pr(request)
-        loop = loop or asyncio.get_event_loop()
+        loop = loop or get_asyncio_loop()
         asyncio.run_coroutine_threadsafe(get_payment_request(), loop)
 
     return out
@@ -1053,16 +1092,29 @@ def create_bip21_uri(addr, amount_sat: Optional[int], message: Optional[str],
     return str(urllib.parse.urlunparse(p))
 
 
-def maybe_extract_bolt11_invoice(data: str) -> Optional[str]:
+def maybe_extract_lightning_payment_identifier(data: str) -> Optional[str]:
     data = data.strip()  # whitespaces
     data = data.lower()
     if data.startswith(LIGHTNING_URI_SCHEME + ':ln'):
-        data = data[10:]
+        cut_prefix = LIGHTNING_URI_SCHEME + ':'
+        data = data[len(cut_prefix):]
     if data.startswith('ln'):
         return data
     return None
 
 
+def is_uri(data: str) -> bool:
+    data = data.lower()
+    if (data.startswith(LIGHTNING_URI_SCHEME + ":") or
+            data.startswith(BITCOIN_BIP21_URI_SCHEME + ':')):
+        return True
+    return False
+
+
+class FailedToParsePaymentIdentifier(Exception):
+    pass
+
+
 # Python bug (http://bugs.python.org/issue1927) causes raw_input
 # to be redirected improperly between stdin/stderr on Unix systems
 #TODO: py3
@@ -1135,6 +1187,7 @@ def read_json_file(path):
         raise FileImportFailed(e)
     return data
 
+
 def write_json_file(path, data):
     try:
         with open(path, 'w+', encoding='utf-8') as f:
@@ -1144,13 +1197,36 @@ def write_json_file(path, data):
         raise FileExportFailed(e)
 
 
+def os_chmod(path, mode):
+    """os.chmod aware of tmpfs"""
+    try:
+        os.chmod(path, mode)
+    except OSError as e:
+        xdg_runtime_dir = os.environ.get("XDG_RUNTIME_DIR", None)
+        if xdg_runtime_dir and is_subpath(path, xdg_runtime_dir):
+            _logger.info(f"Tried to chmod in tmpfs. Skipping... {e!r}")
+        else:
+            raise
+
+
 def make_dir(path, allow_symlink=True):
     """Make directory if it does not yet exist."""
     if not os.path.exists(path):
         if not allow_symlink and os.path.islink(path):
             raise Exception('Dangling link: ' + path)
         os.mkdir(path)
-        os.chmod(path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
+        os_chmod(path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
+
+
+def is_subpath(long_path: str, short_path: str) -> bool:
+    """Returns whether long_path is a sub-path of short_path."""
+    try:
+        common = os.path.commonpath([long_path, short_path])
+    except ValueError:
+        return False
+    short_path = standardize_path(short_path)
+    common     = standardize_path(common)
+    return short_path == common
 
 
 def log_exceptions(func):
@@ -1307,7 +1383,6 @@ class NetworkJobOnDefaultServer(Logger, ABC):
     """
     def __init__(self, network: 'Network'):
         Logger.__init__(self)
-        asyncio.set_event_loop(network.asyncio_loop)
         self.network = network
         self.interface = None  # type: Interface
         self._restart_lock = asyncio.Lock()
@@ -1326,6 +1401,7 @@ class NetworkJobOnDefaultServer(Logger, ABC):
         server connection changes.
         """
         self.taskgroup = OldTaskGroup()
+        self.reset_request_counters()
 
     async def _start(self, interface: 'Interface'):
         self.interface = interface
@@ -1357,6 +1433,13 @@ class NetworkJobOnDefaultServer(Logger, ABC):
             self._reset()
             await self._start(interface)
 
+    def reset_request_counters(self):
+        self._requests_sent = 0
+        self._requests_answered = 0
+
+    def num_requests_sent_and_answered(self) -> Tuple[int, int]:
+        return self._requests_sent, self._requests_answered
+
     @property
     def session(self):
         s = self.interface.session
@@ -1364,9 +1447,41 @@ class NetworkJobOnDefaultServer(Logger, ABC):
         return s
 
 
+_asyncio_event_loop = None  # type: Optional[asyncio.AbstractEventLoop]
+def get_asyncio_loop() -> asyncio.AbstractEventLoop:
+    """Returns the global asyncio event loop we use."""
+    if _asyncio_event_loop is None:
+        raise Exception("event loop not created yet")
+    return _asyncio_event_loop
+
+
 def create_and_start_event_loop() -> Tuple[asyncio.AbstractEventLoop,
                                            asyncio.Future,
                                            threading.Thread]:
+    global _asyncio_event_loop
+    if _asyncio_event_loop is not None:
+        raise Exception("there is already a running event loop")
+
+    # asyncio.get_event_loop() became deprecated in python3.10. (see https://github.com/python/cpython/issues/83710)
+    # We set a custom event loop policy purely to be compatible with code that
+    # relies on asyncio.get_event_loop().
+    # - in python 3.8-3.9, asyncio.Event.__init__, asyncio.Lock.__init__,
+    #   and similar, calls get_event_loop. see https://github.com/python/cpython/pull/23420
+    class MyEventLoopPolicy(asyncio.DefaultEventLoopPolicy):
+        def get_event_loop(self):
+            # In case electrum is being used as a library, there might be other
+            # event loops in use besides ours. To minimise interfering with those,
+            # if there is a loop running in the current thread, return that:
+            running_loop = get_running_loop()
+            if running_loop is not None:
+                return running_loop
+            # Otherwise, return our global loop:
+            return get_asyncio_loop()
+    asyncio.set_event_loop_policy(MyEventLoopPolicy())
+
+    loop = asyncio.new_event_loop()
+    _asyncio_event_loop = loop
+
     def on_exception(loop, context):
         """Suppress spurious messages it appears we cannot control."""
         SUPPRESS_MESSAGE_REGEX = re.compile('SSL handshake|Fatal read error on|'
@@ -1376,13 +1491,21 @@ def create_and_start_event_loop() -> Tuple[asyncio.AbstractEventLoop,
             return
         loop.default_exception_handler(context)
 
-    loop = asyncio.get_event_loop()
+    def run_event_loop():
+        try:
+            loop.run_until_complete(stopping_fut)
+        finally:
+            # clean-up
+            global _asyncio_event_loop
+            _asyncio_event_loop = None
+
     loop.set_exception_handler(on_exception)
     # loop.set_debug(1)
     stopping_fut = loop.create_future()
-    loop_thread = threading.Thread(target=loop.run_until_complete,
-                                         args=(stopping_fut,),
-                                         name='EventLoop')
+    loop_thread = threading.Thread(
+        target=run_event_loop,
+        name='EventLoop',
+    )
     loop_thread.start()
     return loop, stopping_fut, loop_thread
 
@@ -1470,11 +1593,24 @@ def is_ip_address(x: Union[str, bytes]) -> bool:
         return False
 
 
-def is_private_netaddress(host: str) -> bool:
+def is_localhost(host: str) -> bool:
     if str(host) in ('localhost', 'localhost.',):
         return True
     if host[0] == '[' and host[-1] == ']':  # IPv6
         host = host[1:-1]
+    try:
+        ip_addr = ipaddress.ip_address(host)  # type: Union[IPv4Address, IPv6Address]
+        return ip_addr.is_loopback
+    except ValueError:
+        pass  # not an IP
+    return False
+
+
+def is_private_netaddress(host: str) -> bool:
+    if is_localhost(host):
+        return True
+    if host[0] == '[' and host[-1] == ']':  # IPv6
+        host = host[1:-1]
     try:
         ip_addr = ipaddress.ip_address(host)  # type: Union[IPv4Address, IPv6Address]
         return ip_addr.is_private
@@ -1521,10 +1657,10 @@ class CallbackManager:
         self.callbacks = defaultdict(list)      # note: needs self.callback_lock
         self.asyncio_loop = None
 
-    def register_callback(self, callback, events):
+    def register_callback(self, func, events):
         with self.callback_lock:
             for event in events:
-                self.callbacks[event].append(callback)
+                self.callbacks[event].append(func)
 
     def unregister_callback(self, callback):
         with self.callback_lock:
@@ -1538,22 +1674,57 @@ class CallbackManager:
         on the event loop.
         """
         if self.asyncio_loop is None:
-            self.asyncio_loop = asyncio.get_event_loop()
+            self.asyncio_loop = get_asyncio_loop()
             assert self.asyncio_loop.is_running(), "event loop not running"
         with self.callback_lock:
             callbacks = self.callbacks[event][:]
         for callback in callbacks:
             # FIXME: if callback throws, we will lose the traceback
             if asyncio.iscoroutinefunction(callback):
-                asyncio.run_coroutine_threadsafe(callback(event, *args), self.asyncio_loop)
+                asyncio.run_coroutine_threadsafe(callback(*args), self.asyncio_loop)
+            elif get_running_loop() == self.asyncio_loop:
+                # run callback immediately, so that it is guaranteed
+                # to have been executed when this method returns
+                callback(*args)
             else:
-                self.asyncio_loop.call_soon_threadsafe(callback, event, *args)
+                self.asyncio_loop.call_soon_threadsafe(callback, *args)
 
 
 callback_mgr = CallbackManager()
 trigger_callback = callback_mgr.trigger_callback
 register_callback = callback_mgr.register_callback
 unregister_callback = callback_mgr.unregister_callback
+_event_listeners = defaultdict(set)  # type: Dict[str, Set[str]]
+
+
+class EventListener:
+
+    def _list_callbacks(self):
+        for c in self.__class__.__mro__:
+            classpath = f"{c.__module__}.{c.__name__}"
+            for method_name in _event_listeners[classpath]:
+                method = getattr(self, method_name)
+                assert callable(method)
+                assert method_name.startswith('on_event_')
+                yield method_name[len('on_event_'):], method
+
+    def register_callbacks(self):
+        for name, method in self._list_callbacks():
+            _logger.info(f'registering callback {method}')
+            register_callback(method, [name])
+
+    def unregister_callbacks(self):
+        for name, method in self._list_callbacks():
+            _logger.info(f'unregistering callback {method}')
+            unregister_callback(method)
+
+
+def event_listener(func):
+    classname, method_name = func.__qualname__.split('.')
+    assert method_name.startswith('on_event_')
+    classpath = f"{func.__module__}.{classname}"
+    _event_listeners[classpath].add(method_name)
+    return func
 
 
 _NetAddrType = TypeVar("_NetAddrType")
@@ -1623,7 +1794,7 @@ class NetworkRetryManager(Generic[_NetAddrType]):
 class MySocksProxy(aiorpcx.SOCKSProxy):
 
     async def open_connection(self, host=None, port=None, **kwargs):
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         reader = asyncio.StreamReader(loop=loop)
         protocol = asyncio.StreamReaderProtocol(reader, loop=loop)
         transport, _ = await self.create_connection(
@@ -1733,6 +1904,7 @@ class nullcontext:
 
 
 def get_running_loop() -> Optional[asyncio.AbstractEventLoop]:
+    """Returns the asyncio event loop that is *running in this thread*, if any."""
     try:
         return asyncio.get_running_loop()
     except RuntimeError:
